@@ -4,7 +4,14 @@ import datetime
 import errno
 import types
 
-from supervisor.datatypes import signal_number
+from supervisor.compat import as_string
+from supervisor.compat import as_bytes
+from supervisor.compat import unicode
+
+from supervisor.datatypes import (
+    Automatic,
+    signal_number,
+    )
 
 from supervisor.options import readFile
 from supervisor.options import tailFile
@@ -32,6 +39,7 @@ from supervisor.states import getProcessStateDescription
 from supervisor.states import (
     RUNNING_STATES,
     STOPPED_STATES,
+    SIGNALLABLE_STATES
     )
 
 API_VERSION  = '3.0'
@@ -112,8 +120,8 @@ class SupervisorNamespaceRPCInterface:
             raise RPCError(Faults.NO_FILE, logfile)
 
         try:
-            return readFile(logfile, int(offset), int(length))
-        except ValueError, inst:
+            return as_string(readFile(logfile, int(offset), int(length)))
+        except ValueError as inst:
             why = inst.args[0]
             raise RPCError(getattr(Faults, why))
 
@@ -180,7 +188,7 @@ class SupervisorNamespaceRPCInterface:
         self._update('reloadConfig')
         try:
             self.supervisord.options.process_config(do_usage=False)
-        except ValueError, msg:
+        except ValueError as msg:
             raise RPCError(Faults.CANT_REREAD, msg)
 
         added, changed, removed = self.supervisord.diff_to_active()
@@ -227,21 +235,21 @@ class SupervisorNamespaceRPCInterface:
         all_processes = []
 
         if lexical:
-            group_names = self.supervisord.process_groups.keys()
+            group_names = list(self.supervisord.process_groups.keys())
             group_names.sort()
             for group_name in group_names:
                 group = self.supervisord.process_groups[group_name]
-                process_names = group.processes.keys()
+                process_names = list(group.processes.keys())
                 process_names.sort()
                 for process_name in process_names:
                     process = group.processes[process_name]
                     all_processes.append((group, process))
         else:
-            groups = self.supervisord.process_groups.values()
+            groups = list(self.supervisord.process_groups.values())
             groups.sort() # asc by priority
 
             for group in groups:
-                processes = group.processes.values()
+                processes = list(group.processes.values())
                 processes.sort() # asc by priority
                 for process in processes:
                     all_processes.append((group, process))
@@ -283,14 +291,18 @@ class SupervisorNamespaceRPCInterface:
         # eventually fail
         try:
             filename, argv = process.get_execv_args()
-        except NotFound, why:
+        except NotFound as why:
             raise RPCError(Faults.NO_FILE, why.args[0])
-        except (NotExecutable, NoPermission), why:
+        except (NotExecutable, NoPermission) as why:
             raise RPCError(Faults.NOT_EXECUTABLE, why.args[0])
 
         if process.get_state() in RUNNING_STATES:
             raise RPCError(Faults.ALREADY_STARTED, name)
 
+        if process.get_state() == ProcessStates.UNKNOWN:
+            raise RPCError(Faults.FAILED,
+                           "%s is in an unknown process state" % name)
+
         process.spawn()
 
         # We call reap() in order to more quickly obtain the side effects of
@@ -353,7 +365,7 @@ class SupervisorNamespaceRPCInterface:
         if group is None:
             raise RPCError(Faults.BAD_NAME, name)
 
-        processes = group.processes.values()
+        processes = list(group.processes.values())
         processes.sort()
         processes = [ (group, process) for process in processes ]
 
@@ -445,7 +457,7 @@ class SupervisorNamespaceRPCInterface:
         if group is None:
             raise RPCError(Faults.BAD_NAME, name)
 
-        processes = group.processes.values()
+        processes = list(group.processes.values())
         processes.sort()
         processes = [ (group, process) for process in processes ]
 
@@ -494,7 +506,7 @@ class SupervisorNamespaceRPCInterface:
         except ValueError:
             raise RPCError(Faults.BAD_SIGNAL, signal)
 
-        if process.get_state() not in RUNNING_STATES:
+        if process.get_state() not in SIGNALLABLE_STATES:
             raise RPCError(Faults.NOT_RUNNING, name)
 
         msg = process.signal(sig)
@@ -511,17 +523,18 @@ class SupervisorNamespaceRPCInterface:
         @param string signal  Signal to send, as name ('HUP') or number ('1')
         @return array
         """
-        self._update('signalProcessGroup')
 
         group = self.supervisord.process_groups.get(name)
+        self._update('signalProcessGroup')
+
         if group is None:
             raise RPCError(Faults.BAD_NAME, name)
 
-        processes = group.processes.values()
+        processes = list(group.processes.values())
         processes.sort()
         processes = [(group, process) for process in processes]
 
-        sendall = make_allfunc(processes, isRunning, self.signalProcess,
+        sendall = make_allfunc(processes, isSignallable, self.signalProcess,
                                signal=signal)
         result = sendall()
         self._update('signalProcessGroup')
@@ -535,7 +548,7 @@ class SupervisorNamespaceRPCInterface:
         @return array         An array of process status info structs
         """
         processes = self._getAllProcesses()
-        signalall = make_allfunc(processes, isRunning, self.signalProcess,
+        signalall = make_allfunc(processes, isSignallable, self.signalProcess,
             signal=signal)
         result = signalall()
         self._update('signalAllProcesses')
@@ -553,15 +566,39 @@ class SupervisorNamespaceRPCInterface:
         for gconfig in self.supervisord.options.process_group_configs:
             inuse = gconfig.name in self.supervisord.process_groups
             for pconfig in gconfig.process_configs:
-                configinfo.append(
-                    { 'name': pconfig.name,
-                      'group': gconfig.name,
-                      'inuse': inuse,
-                      'autostart': pconfig.autostart,
-                      'group_prio': gconfig.priority,
-                      'process_prio': pconfig.priority })
+                d = {'autostart': pconfig.autostart,
+                     'command': pconfig.command,
+                     'exitcodes': pconfig.exitcodes,
+                     'group': gconfig.name,
+                     'group_prio': gconfig.priority,
+                     'inuse': inuse,
+                     'killasgroup': pconfig.killasgroup,
+                     'name': pconfig.name,
+                     'process_prio': pconfig.priority,
+                     'redirect_stderr': pconfig.redirect_stderr,
+                     'startretries': pconfig.startretries,
+                     'startsecs': pconfig.startsecs,
+                     'stdout_capture_maxbytes': pconfig.stdout_capture_maxbytes,
+                     'stdout_events_enabled': pconfig.stdout_events_enabled,
+                     'stdout_logfile': pconfig.stdout_logfile,
+                     'stdout_logfile_backups': pconfig.stdout_logfile_backups,
+                     'stdout_logfile_maxbytes': pconfig.stdout_logfile_maxbytes,
+                     'stdout_syslog': pconfig.stdout_syslog,
+                     'stopsignal': int(pconfig.stopsignal), # enum on py3
+                     'stopwaitsecs': pconfig.stopwaitsecs,
+                     'stderr_capture_maxbytes': pconfig.stderr_capture_maxbytes,
+                     'stderr_events_enabled': pconfig.stderr_events_enabled,
+                     'stderr_logfile': pconfig.stderr_logfile,
+                     'stderr_logfile_backups': pconfig.stderr_logfile_backups,
+                     'stderr_logfile_maxbytes': pconfig.stderr_logfile_maxbytes,
+                     'stderr_syslog': pconfig.stderr_syslog,
+                    }
+                # no support for these types in xml-rpc
+                d.update((k, 'auto') for k, v in d.items() if v is Automatic)
+                d.update((k, 'none') for k, v in d.items() if v is None)
+                configinfo.append(d)
 
-        configinfo.sort()
+        configinfo.sort(key=lambda r: r['name'])
         return configinfo
 
     def _interpretProcessInfo(self, info):
@@ -672,8 +709,8 @@ class SupervisorNamespaceRPCInterface:
             raise RPCError(Faults.NO_FILE, logfile)
 
         try:
-            return readFile(logfile, int(offset), int(length))
-        except ValueError, inst:
+            return as_string(readFile(logfile, int(offset), int(length)))
+        except ValueError as inst:
             why = inst.args[0]
             raise RPCError(getattr(Faults, why))
 
@@ -806,7 +843,7 @@ class SupervisorNamespaceRPCInterface:
             name = make_namespec(group.config.name, process.config.name)
             try:
                 callback(name)
-            except RPCError, e:
+            except RPCError as e:
                 results.append(
                     {'name':process.config.name,
                      'group':group.config.name,
@@ -844,12 +881,11 @@ class SupervisorNamespaceRPCInterface:
         """
         self._update('sendProcessStdin')
 
-        if isinstance(chars, unicode):
-            chars = chars.encode('utf-8')
-
-        if not isinstance(chars, basestring):
+        if not isinstance(chars, (str, bytes, unicode)):
             raise RPCError(Faults.INCORRECT_PARAMETERS, chars)
 
+        chars = as_bytes(chars)
+
         group, process = self._getGroupAndProcess(name)
 
         if process is None:
@@ -860,7 +896,7 @@ class SupervisorNamespaceRPCInterface:
 
         try:
             process.write(chars)
-        except OSError, why:
+        except OSError as why:
             if why.args[0] == errno.EPIPE:
                 raise RPCError(Faults.NO_FILE, name)
             else:
@@ -914,7 +950,7 @@ def make_allfunc(processes, predicate, func, **extra_kwargs):
                 if predicate(process):
                     try:
                         callback = func(name, **extra_kwargs)
-                    except RPCError, e:
+                    except RPCError as e:
                         results.append({'name':process.config.name,
                                         'group':group.config.name,
                                         'status':e.code,
@@ -939,7 +975,7 @@ def make_allfunc(processes, predicate, func, **extra_kwargs):
 
             try:
                 value = cb()
-            except RPCError, e:
+            except RPCError as e:
                 results.append(
                     {'name':process.config.name,
                      'group':group.config.name,
@@ -987,6 +1023,10 @@ def isRunning(process):
 def isNotRunning(process):
     return not isRunning(process)
 
+def isSignallable(process):
+    if process.get_state() in SIGNALLABLE_STATES:
+        return True
+
 # this is not used in code but referenced via an entry point in the conf file
 def make_main_rpcinterface(supervisord):
     return SupervisorNamespaceRPCInterface(supervisord)
