@@ -20,6 +20,20 @@ else:
 
 class EndToEndTests(BaseTestCase):
 
+    def test_issue_550(self):
+        filename = pkg_resources.resource_filename(__name__, 'fixtures/issue-550.conf')
+        args = ['-m', 'supervisor.supervisord', '-c', filename]
+        supervisord = pexpect.spawn(sys.executable, args, encoding='utf-8')
+        self.addCleanup(supervisord.kill, signal.SIGINT)
+        supervisord.expect_exact('success: print_env entered RUNNING state')
+        supervisord.expect_exact('exited: print_env (exit status 0; expected)')
+
+        args = ['-m', 'supervisor.supervisorctl', '-c', filename, 'tail -100000', 'print_env']
+        supervisorctl = pexpect.spawn(sys.executable, args, encoding='utf-8')
+        self.addCleanup(supervisorctl.kill, signal.SIGINT)
+        supervisorctl.expect_exact("THIS_SHOULD=BE_IN_CHILD_ENV")
+        supervisorctl.expect(pexpect.EOF)
+
     def test_issue_565(self):
         filename = pkg_resources.resource_filename(__name__, 'fixtures/issue-565.conf')
         args = ['-m', 'supervisor.supervisord', '-c', filename]
@@ -129,6 +143,27 @@ class EndToEndTests(BaseTestCase):
             seen = False
         self.assertTrue(seen)
 
+    def test_issue_1170a(self):
+        filename = pkg_resources.resource_filename(__name__, 'fixtures/issue-1170a.conf')
+        args = ['-m', 'supervisor.supervisord', '-c', filename]
+        supervisord = pexpect.spawn(sys.executable, args, encoding='utf-8')
+        self.addCleanup(supervisord.kill, signal.SIGINT)
+        supervisord.expect_exact("set from [supervisord] section")
+
+    def test_issue_1170b(self):
+        filename = pkg_resources.resource_filename(__name__, 'fixtures/issue-1170b.conf')
+        args = ['-m', 'supervisor.supervisord', '-c', filename]
+        supervisord = pexpect.spawn(sys.executable, args, encoding='utf-8')
+        self.addCleanup(supervisord.kill, signal.SIGINT)
+        supervisord.expect_exact("set from [program] section")
+
+    def test_issue_1170c(self):
+        filename = pkg_resources.resource_filename(__name__, 'fixtures/issue-1170c.conf')
+        args = ['-m', 'supervisor.supervisord', '-c', filename]
+        supervisord = pexpect.spawn(sys.executable, args, encoding='utf-8')
+        self.addCleanup(supervisord.kill, signal.SIGINT)
+        supervisord.expect_exact("set from [eventlistener] section")
+
     def test_issue_1224(self):
         filename = pkg_resources.resource_filename(__name__, 'fixtures/issue-1224.conf')
         args = ['-m', 'supervisor.supervisord', '-c', filename]
@@ -206,6 +241,21 @@ class EndToEndTests(BaseTestCase):
                                  timeout=30)
             supervisorctl.expect('Unicode output may fail.', timeout=30)
 
+    def test_issue_1251(self):
+        args = ['-m', 'supervisor.supervisord', '-?']
+        supervisord = pexpect.spawn(sys.executable, args, encoding='utf-8')
+        self.addCleanup(supervisord.kill, signal.SIGINT)
+        supervisord.expect_exact("supervisord -- run a set of applications")
+        supervisord.expect_exact("-l/--logfile FILENAME -- use FILENAME as")
+        supervisord.expect(pexpect.EOF)
+
+        args = ['-m', 'supervisor.supervisorctl', '-?']
+        supervisorctl = pexpect.spawn(sys.executable, args, encoding='utf-8')
+        self.addCleanup(supervisorctl.kill, signal.SIGINT)
+        supervisorctl.expect_exact("supervisorctl -- control applications")
+        supervisorctl.expect_exact("-i/--interactive -- start an interactive")
+        supervisorctl.expect(pexpect.EOF)
+
     def test_issue_1298(self):
         filename = pkg_resources.resource_filename(__name__, 'fixtures/issue-1298.conf')
         args = ['-m', 'supervisor.supervisord', '-c', filename]
